/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Data;
using System.IO;

namespace Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility {

	/// <summary>
	/// This class contains some useful util methods for reading and transforming
	/// XML files and authentication a user
	/// </summary>
	public class XmlUtility {

		private DataSet XmlToDataSet(String fileName) {

			DataSet ds = new DataSet();
			FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read);
			StreamReader reader = new StreamReader(fs);
			ds.ReadXml(reader);
			fs.Close();
			return ds;
		}

		private ArrayList GetValuesFromTable(DataTable table, String filter) {

			ArrayList valuesList = new ArrayList();
			DataRow[] rows;
			if(filter == null) {
				rows = table.Select();
			}
			else {
				rows = table.Select(filter);
			}
			for(int i = 0; i < rows.Length; i++) {
				DataRow row = rows[i];
				Hashtable dataTable = new Hashtable();
				for(int j = 0; j < table.Columns.Count; j++) {
					String key   = table.Columns[j].ToString();
					String value = (String)row[key];
					dataTable.Add(key, value);
				}
				valuesList.Add(dataTable);
			}
			return valuesList;
		}

		public ArrayList GetPropsFromXmlFile(String fileName) {

			DataSet ds = XmlToDataSet(fileName);
			DataTable dataTable = ds.Tables[0];
			ArrayList propsList = GetValuesFromTable(dataTable, null);
			return propsList;
		}

		public Hashtable GetConfigProps(String configPath) {

			Hashtable configTable = null;
			ArrayList configList = GetPropsFromXmlFile(configPath);
			if(configList.Count > 0) {
				configTable = (Hashtable)configList[0];
			}
			return configTable;
		}

		public Hashtable AuthenticateUser(String userName, String password,
				String usersFile) {
			
			Hashtable userInfo = null;

			DataSet ds = XmlToDataSet(usersFile);
			DataTable usersTable = ds.Tables[0];

			String userFilter = Constants.USER_ID + "=" + "'" + userName + "'";
			ArrayList usersList = GetValuesFromTable(usersTable, userFilter);
			if(usersList.Count > 0) {
				for(int i = 0; i < usersList.Count; i++) {
					Hashtable user = (Hashtable)usersList[i];
					if( !user.ContainsKey(Constants.USER_PASSWORD) ) {
						throw new NoSuchTagException("Invalid Tag Name for User Password");
					}
					String userPassword = (String)user[Constants.USER_PASSWORD];
					if( userPassword.Equals(password) ) {
						userInfo = user;
						break;
					}
				}
			}
			return userInfo;
		}
	}
}